/*
 * Copyright (C) 2001-2003 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * -
 * Portions Copyright (C) 2001 Jesper Hansen <jesperh@telia.com>.
 * 
 * This file is part of the yampp system.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * $Log: vs1001k.c,v $
 * Revision 1.9  2003/02/14 13:39:00  mac
 * Several serious bugs fixed,
 * interrupt routine completely remade.
 * Unreliable spurious interrupts detection removed.
 * Mpeg frame detection removed.
 * Watermark check removed (this was rather limiting)
 * Can be optionaly compiled not to use SPI
 *
 * Revision 1.8  2003/02/04 17:50:55  harald
 * Version 3 released
 *
 * Revision 1.7  2003/01/14 16:15:19  harald
 * Sending twice the number of zeros to end MP3 stream.
 * Check for spurious interrupts to detect hanging chip.
 * Simpler portable inline assembler for short delays.
 *
 * Revision 1.6  2002/11/02 15:15:13  harald
 * Library dependencies removed
 *
 * Revision 1.5  2002/09/15 16:44:14  harald
 * *** empty log message ***
 *
 * Revision 1.4  2002/08/16 17:49:02  harald
 * First public release
 *
 * Revision 1.3  2002/06/26 17:29:08  harald
 * First pre-release with 2.4 stack
 *
 */

#include <sys/atom.h>
#include <sys/event.h>
#include <sys/timer.h>
#include <sys/heap.h>

#include <dev/irqreg.h>

#include "vs1001k.h"

/*!
 * \addtogroup xgVs1001
 */
/*@{*/


static u_char writeDelay = 0;

static volatile u_char vs_status=VS_STATUS_STOPPED;
static volatile u_short vs_flush=0;

static u_char *vs_databuff;
static u_char *vs_dataend;

static u_char *vs_writeptr;
static volatile u_char *vs_readptr;


#define VsEnterCritical() cbi(EIMSK, 6)
#define VsExitCritical() sbi(EIMSK, 6)

/*
 * Write a byte to the VS1001 data interface.
 */
static INLINE void VsSdiPutByte(u_char b)
{
#ifdef VS_NOSPI
    u_char mask=0x80;
    sbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    while (mask) {
      if (b&mask) sbi(VS_SI_PORT, VS_SI_BIT);
      else cbi(VS_SI_PORT, VS_SI_BIT);

      sbi(VS_SS_PORT, VS_SS_BIT);
      mask>>=1;
      cbi(VS_SS_PORT, VS_SS_BIT);
      cbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    }
#else
    sbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    outp(inp(SPSR), SPSR); /* clear SPI interrupt flag */
    outp(b, SPDR);
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    cbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    loop_until_bit_is_set(SPSR, SPIF); /* wait for SPI transfer to finish */
#endif
}

/*!
 * \brief Write a byte to the serial control interface.
 */
static INLINE void VsSciPutByte(u_char data)
{
    u_char mask = 0x80;

    /*
     * Loop until all 8 bits are processed.
     */
    while (mask) {

        /*
         * Set data line.
         */
        if (data & mask)
            sbi(VS_SI_PORT, VS_SI_BIT);
        else
            cbi(VS_SI_PORT, VS_SI_BIT);

        /*
         * Toggle clock and shift mask.
         */
        sbi(VS_SCK_PORT, VS_SCK_BIT);
        mask >>= 1;
        cbi(VS_SCK_PORT, VS_SCK_BIT);
    }
}

/*!
 * \brief Read a byte from the serial control interface.
 */
static INLINE u_char VsSciGetByte(void)
{
    u_char mask = 0x80;
    u_char data = 0;

    /*
     * Loop until all 8 bits are processed.
     */
    while (mask) {

        /*
         * Toggle clock and get the data.
         */
        sbi(VS_SCK_PORT, VS_SCK_BIT);
        if (bit_is_set(VS_SO_PIN,VS_SO_BIT))
        data|=mask;
        mask >>= 1;
        cbi(VS_SCK_PORT, VS_SCK_BIT);
    }
    return data;
}

/*
 * Write to a register.
 */
static void VsRegWrite(u_char reg, u_short data)
{
    /*
     * Disable interrupts and select chip.
     */
    VsEnterCritical();
    cbi(VS_XCS_PORT, VS_XCS_BIT);

    /*
     * Disable SPI
     */
#ifndef VS_NOSPI
    cbi(SPCR, SPE);
    //outp(BV(MSTR) | BV(SPE), SPCR);
#endif

    VsSciPutByte(VS_OPCODE_WRITE);
    VsSciPutByte(reg);
    VsSciPutByte((u_char) (data >> 8));
    VsSciPutByte((u_char) data);

    /*
     * Added on Jesper's recommendation.
     */
    if (writeDelay)
        NutDelay(writeDelay);

    /* 
     * Re-enable SPI. Changed due to a hint by Jesper.
     */
#ifndef VS_NOSPI
    outp(BV(MSTR) | BV(SPE), SPCR);
    outp(inp(SPSR), SPSR);
#endif

    /*
     * Deselect chip and enable interrupts.
     */
    sbi(VS_XCS_PORT, VS_XCS_BIT);
    VsExitCritical();
}

/*
 * Read from a register.
 */
static u_short VsRegRead(u_char reg)
{
    u_short data;
    /*
     * Disable interrupts and select chip.
     */
    VsEnterCritical();
    cbi(VS_XCS_PORT, VS_XCS_BIT);

    /*
     * Disable SPI
     */
#ifndef VS_NOSPI
    cbi(SPCR, SPE);
    //outp(BV(MSTR) | BV(SPE), SPCR);
#endif

    VsSciPutByte(VS_OPCODE_READ);
    VsSciPutByte(reg);
    data=VsSciGetByte()<<8;
    data|=VsSciGetByte();

    /*
     * Re-enable SPI. Changed due to a hint by Jesper.
     */
#ifndef VS_NOSPI
    outp(BV(MSTR) | BV(SPE), SPCR);
    outp(inp(SPSR), SPSR);
#endif

    /*
     * Deselect chip and enable interrupts.
     */
    sbi(VS_XCS_PORT, VS_XCS_BIT);
    VsExitCritical();

    return data;
}

/*
 * Data request interrupt service.
 */
static void VsPlayerFeed(void *arg)
{
    u_char j;

    /* cancel interrupt if not running */
    if (vs_status!=VS_STATUS_RUNNING) return;
    
    /* feed the decoder until its buffer is full */
    while (bit_is_set(VS_DREQ_PIN, VS_DREQ_BIT)) {
      /* we may feed 32 bytes block at a time */
      for (j=32; j--;) {
        /* is main buffer empty? */
        if (vs_readptr==vs_writeptr) {
          if (vs_flush) {
            /* auto flush the internal VS buffer */
            VsSdiPutByte(0);
            if (!(--vs_flush)) vs_status=VS_STATUS_EMPTY;
          }
          else {
            /* end of stream */
            vs_status=VS_STATUS_EOF;
            return;
          }
        }
        /* we have some data in the buffer, feed it */
        else {
          VsSdiPutByte(*vs_readptr);
          if (++vs_readptr==vs_dataend) vs_readptr=vs_databuff;
        }
      }
    }
}

/*!
 * \brief Start playback.
 *
 * This routine will send the first MP3 data bytes to the
 * decoder, until it is completely filled. The data buffer
 * should have been filled with at least about 6k of data
 * before calling this routine.
 *
 * If called for the first time, the routine will register
 * the interrupt handler.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerKick(void)
{
    VsEnterCritical();

    /*
     * Start feeding the decoder with data and
     * enable interrupts.
     */
    vs_status = VS_STATUS_RUNNING;
    VsPlayerFeed(NULL);

    VsExitCritical();
    return 0;
}

/*!
 * \brief Stops the playback.
 *
 * This routine will stops the MP3 playback,
 * VsPlayerKick() may be used to resume the playback.
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerStop(void)
{
  VsEnterCritical();
  /* check whether we need to stop at all to not overwrite other than running status */
  if (vs_status == VS_STATUS_RUNNING) vs_status = VS_STATUS_STOPPED;
  VsExitCritical();
  return 0;
}


/*!
 * \brief Sets up VS internal buffer flushing.
 *
 * This routine will set up internal VS buffer flushing,
 * unless the buffer is already empty and starts the playback
 * if necessary. The internal VS buffer is flushed in VsPlayerFeed()
 * at the end of the stream.
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerFlush(void)
{
  VsEnterCritical();
  /* set up fluhing unless both buffers are empty */
  if (vs_status!=VS_STATUS_EMPTY || vs_readptr!=vs_writeptr) {
    if (!vs_flush) vs_flush=VS_FLUSH_BYTES;
    /* start the palyback if necessary */
    if (vs_status!=VS_STATUS_RUNNING) VsPlayerKick();
  }
  VsExitCritical();
  return 0;
}


/*!
 * \brief Initialize the VS1001 hardware interface.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerInit(void)
{
    writeDelay = 0;

    /*
     * Toggle MP3 hardware reset output.
     */
    cbi(VS_RESET_PORT, VS_RESET_BIT);
    sbi(VS_RESET_DDR, VS_RESET_BIT);
    NutDelay(3);
    sbi(VS_RESET_PORT, VS_RESET_BIT);

    /*
     * Set BSYNC output low.
     */
    cbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    sbi(VS_BSYNC_DDR, VS_BSYNC_BIT);

    /*
     * Set MP3 chip select output low.
     */
    sbi(VS_XCS_PORT, VS_XCS_BIT);
    sbi(VS_XCS_DDR, VS_XCS_BIT);

    /*
     * Set DREQ input with pullup.
     */
    cbi(VS_DREQ_DDR, VS_DREQ_BIT);
    sbi(VS_DREQ_PORT, VS_DREQ_BIT);

    /*
     * Init SPI Port.
     */
    sbi(VS_SI_DDR, VS_SI_BIT);
    sbi(VS_SS_DDR, VS_SS_BIT);
    cbi(VS_SO_DDR, VS_SO_BIT);

    /*
     * Set SCK output low.
     */
    sbi(VS_SCK_DDR, VS_SCK_BIT);
    cbi(VS_SCK_PORT, VS_SCK_BIT);

    /*
     * Init SPI mode.
     */
#ifndef VS_NOSPI
    outp(BV(MSTR) | BV(SPE), SPCR);
#ifdef __IMAGECRAFT__
    {
        u_char dummy = inp(SPSR);
    }
#else
    inp(SPSR);
#endif
#endif
    NutDelay(2);

    /*
     * Register the interrupt routine
     */
    VsEnterCritical();
    sbi(EICR, 5);
    sbi(EICR, 4);
    sbi(EIFR, 6); /* clear spurious interrupt */
    NutRegisterIrqHandler(&sig_INTERRUPT6, VsPlayerFeed, NULL);
    VsExitCritical();

    return 0;
}

/*!
 * \brief Software reset the decoder.
 *
 * This function is typically called after VsPlayerInit().
 *
 * \param mode Any of the following flags may be or'ed
 * - VS_SM_DIFF
 * - VS_SM_FFWD
 * - VS_SM_RESET
 * - VS_SM_MP12
 * - VS_SM_PDOWN
 * - VS_SM_DAC
 * - VS_SM_DACMONO
 * - VS_SM_BASS
 * - VS_SM_DACT
 * - VS_SM_BYTEORD
 * - VS_SM_IBMODE
 * - VS_SM_IBCLK
 *
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerReset(u_short mode)
{
    VsEnterCritical();
    /*
     * Software reset, set modes of decoder.
     */
    VsRegWrite(VS_MODE_REG, VS_SM_RESET|mode);
    NutDelay(2);

    /*
     * Expect DREQ.
     */
    if (bit_is_clear(VS_DREQ_PIN, VS_DREQ_BIT)) {
      /* if not succeeded, give it one more chance and try hw reset */
      sbi(VS_RESET_DDR, VS_RESET_BIT);
      NutDelay(3);
      sbi(VS_RESET_PORT, VS_RESET_BIT);

      /* set the requested modes */
      VsRegWrite(VS_MODE_REG, VS_SM_RESET|mode);
      NutDelay(2);
      if (bit_is_clear(VS_DREQ_PIN, VS_DREQ_BIT)) return -1;
    }

    VsRegWrite(VS_CLOCKF_REG, 0x9800);
    VsRegWrite(VS_INT_FCTLH_REG, 0x8008); /* force frequency change (see datasheet) */

    /*
     * At least one zero should be sent to SDI according to datasheet
     */
    VsSdiPutByte(0);
    VsExitCritical();

    return 0;
}

/*!
 * \brief Sets mode register of the decoder.
 */
int VsPlayerSetMode(u_short mode)
{
  VsRegWrite(VS_MODE_REG, mode);
  return 0;
}

/*!
 * \brief Initialize the MP3 data buffer.
 *
 * \param size Number of bytes to allocate for the data buffer.
 *             Should be at least 4k. If this parameter is 0,
 *             all available memory minus 8k are allocated.
 *
 * \return Pointer to the data buffer or null on failures.
 */
u_char *VsBufferInit(u_short size)
{
    if (size == 0)
        size = NutHeapAvailable() - 8192;
    if ((vs_databuff = NutHeapAlloc(size)) != 0)
        vs_dataend = vs_databuff + size;

    vs_readptr = vs_writeptr = vs_databuff;
    return vs_writeptr;
}

/*!
 * \brief Reset all MP3 data buffer pointers.
 *
 * \return Pointer to the data buffer.
 */
u_char *VsBufferReset(void)
{
    VsEnterCritical();
    vs_readptr = vs_writeptr = vs_databuff;
    VsExitCritical();

    return vs_databuff;
}

/*!
 * \brief Request MP3 data buffer space.
 *
 * \param sizep Pointer to an unsigned short, which receives the
 *              number of bytes available in the buffer.
 *
 * \return Pointer to the next write position.
 */
u_char *VsBufferRequest(u_short * sizep)
{
    VsEnterCritical();
    if (vs_writeptr >= vs_readptr)
        *sizep = vs_dataend - vs_writeptr - (vs_readptr==vs_databuff);
    else
        *sizep = vs_readptr - vs_writeptr - 1;
    VsExitCritical();

    return vs_writeptr;
}

/*!
 * \brief Acknowledge filled buffer space.
 *
 * \return Pointer to the next write position.
 */
u_char *VsBufferAcknowledge(u_short nbytes)
{
    VsEnterCritical();
    vs_writeptr += nbytes;
    if (vs_writeptr == vs_dataend)
        vs_writeptr = vs_databuff;
    vs_flush = 0; /* cancel flushing in progress */
    VsExitCritical();

    return vs_writeptr;
}


/*!
 * \brief Returns total free buffer space.
 *
 * \return Total number of free bytes in the buffer.
 */
u_short VsBufferAvailable(void)
{
    u_short free;
    VsEnterCritical();
    if (vs_writeptr>=vs_readptr)
      free = (vs_dataend-vs_databuff) - (vs_writeptr-vs_readptr) - 1;
    else
      free = (vs_readptr-vs_writeptr) - 1;
    VsExitCritical();
    return free;
}

/*!
 * \brief Returns play time since last reset.
 *
 * \return play time since reset in seconds
 */
u_short VsPlayTime(void)
{
  return VsRegRead(VS_DECODE_TIME_REG);
}

/*!
 * \brief Returns status of the player.
 *
 * \return 1=playing, 0=stopped/finished
 */
u_char VsGetStatus(void)
{
  return vs_status;
}

/*!
 * \brief Set volume.
 *
 * \param left  Left channel volume.
 * \param right Right channel volume.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsSetVolume(u_char left, u_char right)
{
    VsRegWrite(VS_VOL_REG, (((u_short) left) << 8) | (u_short) right);
    return 0;
}

/*!
 * \brief Sine wave beep.
 *
 * \param fsin Frequency.
 * \param ms   Duration.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsBeep(u_char fsin, u_char ms)
{
    u_char i;
    u_char on[] = { 0x53, 0xEF, 0x6E, 56 };
    u_char off[] = { 0x45, 0x78, 0x69, 0x74 };

    on[3] += (fsin & 7) * 9;
    for (i = 0; i < sizeof(on); i++)
        VsSdiPutByte(on[i]);
    for (i = 0; i < 4; i++)
        VsSdiPutByte(0);
    NutDelay(ms);
    for (i = 0; i < sizeof(off); i++)
        VsSdiPutByte(off[i]);
    for (i = 0; i < 4; i++)
        VsSdiPutByte(0);

    return 0;
}


/*@}*/
