/*
 * Copyright (C) 2001-2003 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * -
 * Portions Copyright (C) 2001 Jesper Hansen <jesperh@telia.com>.
 * 
 * This file is part of the yampp system.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * $Log: vs1001k.c,v $
 * Revision 1.8  2003/02/04 17:50:55  harald
 * Version 3 released
 *
 * Revision 1.7  2003/01/14 16:15:19  harald
 * Sending twice the number of zeros to end MP3 stream.
 * Check for spurious interrupts to detect hanging chip.
 * Simpler portable inline assembler for short delays.
 *
 * Revision 1.6  2002/11/02 15:15:13  harald
 * Library dependencies removed
 *
 * Revision 1.5  2002/09/15 16:44:14  harald
 * *** empty log message ***
 *
 * Revision 1.4  2002/08/16 17:49:02  harald
 * First public release
 *
 * Revision 1.3  2002/06/26 17:29:08  harald
 * First pre-release with 2.4 stack
 *
 */

#include <sys/atom.h>
#include <sys/event.h>
#include <sys/timer.h>
#include <sys/heap.h>

#include <dev/irqreg.h>

#include "vs1001k.h"
#include "timer.h"

/*!
 * \addtogroup xgVs1001
 */
/*@{*/


static u_char writeDelay = 0;
static u_char registered = 0;

//static volatile u_char vs_running;
//static volatile u_char vs_inplay;
static volatile u_char vs_failed;

static u_char *vs_databuff;
static u_char *vs_dataend;
//static volatile u_short vs_datacnt; //!!duplicated information is not needed
//static u_short vs_watermark;
static volatile u_char *vs_writeptr;
static volatile u_char *vs_readptr;

//static volatile u_char vs_feed;

/*
 * Write a byte to the VS1001 data interface.
 */
void VsSdiPutByte(u_char b)
{
  u_char mask=0x80;
  sbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
  while (mask) {
    if (b&mask) sbi(VS_SI_PORT, VS_SI_BIT);
    else cbi(VS_SI_PORT, VS_SI_BIT);

    sbi(VS_SS_PORT, VS_SS_BIT);
    mask>>=1;
    cbi(VS_SS_PORT, VS_SS_BIT);
    cbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
  }

/*
    sbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    outp(*vs_readptr, SPDR);
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    _NOP();
    cbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
*/
}

/*!
 * \brief Write a byte to the serial control interface.
 */
static INLINE void VsSciPutByte(u_char data)
{
    u_char mask = 0x80;

    /*
     * Loop until all 8 bits are processed.
     */
    while (mask) {

        /*
         * Set data line.
         */
        if (data & mask)
            sbi(VS_SI_PORT, VS_SI_BIT);
        else
            cbi(VS_SI_PORT, VS_SI_BIT);

        /*
         * Toggle clock and shift mask.
         */
        sbi(VS_SCK_PORT, VS_SCK_BIT);
        mask >>= 1;
        cbi(VS_SCK_PORT, VS_SCK_BIT);
    }
}

/*
 * Write a word to the VS1001 command interface.
 */
void VsRegWrite(u_char reg, u_short data)
{
    /*
     * Disable interrupts and select chip.
     */
    NutEnterCritical(); //!! is it necessary tu turn of ALL interrupts?
    cbi(VS_XCS_PORT, VS_XCS_BIT);

    //cbi(SPCR, SPE); //!! SPI not used for now
    //outp(BV(MSTR) | BV(SPE), SPCR);

    VsSciPutByte(VS_OPCODE_WRITE);
    VsSciPutByte(reg);
    VsSciPutByte((u_char) (data >> 8));
    VsSciPutByte((u_char) data);

    /*
     * Added on Jesper's recommendation.
     */
    if (writeDelay)
        NutDelay(writeDelay);

    /* 
     * Re-enable SPI. Changed due to a hint by Jesper.
     */
    //outp(BV(MSTR) | BV(SPE), SPCR);
    //outp(inp(SPSR), SPSR);

    /*
     * Deselect chip and enable interrupts.
     */
    sbi(VS_XCS_PORT, VS_XCS_BIT);
    NutExitCritical();
}

/*
 * Data request interrupt service.
 *
 * The routine serves two purposes. It's an interrupt
 * service routine and is used to start the playback.
 */
static void VsPlayerFeed(void *arg)
{
    u_char j;

    //if ((vs_feed=!vs_feed)) return;
    
    /*
     * Spurious interrupts most probably occure when bad MP3
     * data has been fed to the decoder.
     */
/*
    if (vs_failed || bit_is_clear(PINE, 6)) {
        vs_running = 0;
        vs_failed = 1;
        cbi(EIMSK, 6);
        return;
    }
*/
/*
    for (;;) {
        if (bit_is_clear(PINE, 6)) {
            vs_running = 1;
            vs_inplay = 1;
            break;
        }
        VsSdiPutByte(*vs_readptr);
        if (++vs_readptr == vs_dataend)
            vs_readptr = vs_databuff;
        vs_datacnt--;
        loop_until_bit_is_set(SPSR, SPIF);
*/

        /*
         * Check if we ran out of data.
         */
/*
        if (vs_datacnt == 0) {
            vs_running = 0;
            cbi(EIMSK, 6);
            return;
        }
    }
*/
    /*
     * The decoder allows to add addional 32 data bytes.
     */
/*
    if (vs_datacnt >= 32) {
        for (j = 32; j--;) {
            VsSdiPutByte(*vs_readptr);
            if (++vs_readptr == vs_dataend)
                vs_readptr = vs_databuff;
            loop_until_bit_is_set(SPSR, SPIF);
        }
        vs_datacnt -= 32;
    }
*/
    while (bit_is_set(PINE, 6)) {
      for (j=32; j--;) {
        if (vs_readptr==vs_writeptr) return; //!!buffer underrun - handle it
        VsSdiPutByte(*vs_readptr);
        if (++vs_readptr==vs_dataend)
          vs_readptr=vs_databuff;
          //loop_until_bit_is_set(SPSR, SPIF); //!!SPI is not used for now
      }
    }
}

/*!
 * \brief Start playback.
 *
 * This routine will send the first MP3 data bytes to the
 * decoder, until it is completely filled. The data buffer
 * should have been filled with at least about 6k of data
 * before calling this routine.
 *
 * If called for the first time, the routine will register
 * the interrupt handler.
 *
 * If the decoder is activated, 2048 nulls are sent first.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerKick(void)
{
  u_char j;
/*
    int rc = 0;
    u_short i;
    if (vs_failed) {
        cbi(VS_RESET_PORT, VS_RESET_BIT);
        NutDelay(3);
        sbi(VS_RESET_PORT, VS_RESET_BIT);
        return -1;
    }

    if (vs_running || vs_datacnt < vs_watermark)
        return 0;

    if (vs_inplay) {
        for (i = 0; i < 2048; i++) {
            loop_until_bit_is_set(PINE, 6);
            VsSdiPutByte(0);
        }
    }
*/
    /*
     * Search next mp3 header.
     */
/*
    while (vs_datacnt > 2) {
        if (*vs_readptr == 0xFF &&
            *(vs_readptr + 1) != 0xFF &&
            (*(vs_readptr + 1) & 0xFE) == 0xFA)
            break;
        if (++vs_readptr == vs_dataend)
            vs_readptr = vs_databuff;
        vs_datacnt--;
    }
    if (vs_datacnt <= 2)
        return -1;
*/
    /*
     * Register the interrupt routine, if not
     * done yet.
     */
    cbi(EIMSK, 6);
    if (!registered) {
        NutRegisterIrqHandler(&sig_INTERRUPT6, VsPlayerFeed, 0);
        sbi(EICR, 5); //rising edge will trigger the interrupt
        sbi(EICR, 4);
        sbi(EIFR, 6);

        //NutRegisterInterrupt(IRQ_TIMER2_OVF, VsPlayerFeed, 0); // Nut/OS <3
        //NutRegisterIrqHandler(&sig_TIMER2_OVF, VsPlayerFeed, 0);
        //outp(0x05,TCCR2);
        //sbi(TIFR, 6);
        //sbi(TIMSK, 6);
        registered = 1;
    }

    /*
     * Start feeding the decoder with data and
     * enable interrupts.
     */
    while (bit_is_set(PINE, 6)) {
      for (j=32; j--;) {
        if (vs_readptr==vs_writeptr) return 0; //!!buffer underrun - handle it
        VsSdiPutByte(*vs_readptr);
        if (++vs_readptr==vs_dataend)
          vs_readptr=vs_databuff;
          //loop_until_bit_is_set(SPSR, SPIF); //SPI is not used for now
      }
    }
    sbi(EIMSK, 6);

    return 0;
}

/*!
 * \brief Initialize the VS1001 hardware interface.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerInit(void)
{
    writeDelay = 0;

    /*
     * Toggle MP3 hardware reset output.
     */
    cbi(VS_RESET_PORT, VS_RESET_BIT);
    sbi(VS_RESET_DDR, VS_RESET_BIT);
    NutDelay(3);
    sbi(VS_RESET_PORT, VS_RESET_BIT);

    /*
     * Set BSYNC output low.
     */
    cbi(VS_BSYNC_PORT, VS_BSYNC_BIT);
    sbi(VS_BSYNC_DDR, VS_BSYNC_BIT);

    /*
     * Set MP3 chip select output high (inactive).
     */
    sbi(VS_XCS_PORT, VS_XCS_BIT);
    sbi(VS_XCS_DDR, VS_XCS_BIT);

    /*
     * Set DREQ input with pullup.
     */
    cbi(VS_DREQ_DDR, VS_DREQ_BIT);
    sbi(VS_DREQ_PORT, VS_DREQ_BIT);

    /*
     * Init SPI Port.
     */
    sbi(VS_SI_DDR, VS_SI_BIT);
    sbi(VS_SS_DDR, VS_SS_BIT);
    cbi(VS_SO_DDR, VS_SO_BIT);

    /*
     * Set SCK output low.
     */
    sbi(VS_SCK_DDR, VS_SCK_BIT);
    cbi(VS_SCK_PORT, VS_SCK_BIT);

    /*
     * Init SPI mode.
     */
/*
    outp(BV(MSTR) | BV(SPE), SPCR);
#ifdef __IMAGECRAFT__
    {
        u_char dummy = inp(SPSR);
    }
#else
    inp(SPSR);
#endif
*/
    return 0;
}

/*!
 * \brief Software reset the decoder.
 *
 * This function is typically called after VsPlayerInit().
 *
 * \param mode Any of the following flags may be or'ed
 * - VS_SM_DIFF
 * - VS_SM_FFWD
 * - VS_SM_RESET
 * - VS_SM_MP12
 * - VS_SM_PDOWN
 * - VS_SM_DAC
 * - VS_SM_DACMONO
 * - VS_SM_BASS
 * - VS_SM_DACT
 * - VS_SM_BYTEORD
 * - VS_SM_IBMODE
 * - VS_SM_IBCLK
 *
 * \return 0 on success, -1 otherwise.
 */
int VsPlayerReset(u_short mode)
{
    vs_failed = 0;

    /*
     * Software reset.
     */
    VsRegWrite(VS_MODE_REG, VS_SM_RESET);
    NutMicroDelay(2); //!!!microseconds here

    /*
     * Wait for DREQ.
     */
    //loop_until_bit_is_set(VS_DREQ_PIN, VS_DREQ_BIT); //!!!this may hang
    //NutDelay(200); //!!!this should be usec

    NutDelay(1);
    if (bit_is_clear(VS_DREQ_PIN, VS_DREQ_BIT)) return -1;
    
    VsRegWrite(VS_CLOCKF_REG, 0x9800);
    VsRegWrite(VS_INT_FCTLH_REG, 0x8008);

    /*
     * Set configured modes
     */
    VsRegWrite(VS_MODE_REG, mode);
    VsSdiPutByte(0); //at least one zero should be sent according to datasheet
    
    return 0;
}


/*!
 * \brief Initialize the MP3 data buffer.
 *
 * \param size Number of bytes to allocate for the data buffer.
 *             Should be at least 4k. If this parameter is 0,
 *             all available memory minus 8k are allocated.
 *
 * \return Pointer to the data buffer or null on failures.
 */
u_char *VsBufferInit(u_short size)
{
    if (size == 0)
        size = NutHeapAvailable() - 8192;
    //vs_watermark = size - 1024;
    if ((vs_databuff = NutHeapAlloc(size)) != 0)
        vs_dataend = vs_databuff + size;

    vs_readptr = vs_writeptr = vs_databuff; //also reset the buffer
    return vs_databuff;
}

/*!
 * \brief Reset all MP3 data buffer pointers.
 *
 * \return Pointer to the data buffer.
 */
u_char *VsBufferReset(void)
{
    //!!disable interrupt first
    vs_readptr = vs_writeptr = vs_databuff;
    //vs_datacnt = 0;

    return vs_databuff;
}

/*!
 * \brief Request MP3 data buffer space.
 *
 * \param sizep Pointer to an unsigned short, which receives the
 *              number of continuous bytes available in the buffer.
 *
 * \return Pointer to the next write position.
 */
u_char *VsBufferRequest(u_short *sizep)
{
    cbi(EIMSK, 6);
    u_char *loc_vs_readptr=(char *)vs_readptr; //local copy of volatile variable
    if (vs_writeptr>=loc_vs_readptr) {
        *sizep=vs_dataend-vs_writeptr-(loc_vs_readptr==vs_databuff); //at least one byte must remain free
    }
    else {
        *sizep=loc_vs_readptr-vs_writeptr-1;
    }
    //if (vs_running)
        sbi(EIMSK, 6);

    return (char *)vs_writeptr;
}

/*!
 * \brief Acknowledge filled buffer space.
 *
 * \return Pointer to the next write position.
 */
u_char *VsBufferAcknowledge(u_short nbytes)
{
    cbi(EIMSK, 6);
    //vs_datacnt += nbytes;
    if (vs_writeptr+nbytes == vs_dataend)
      vs_writeptr = vs_databuff;
    else
      vs_writeptr += nbytes;
    //if (vs_running)
    sbi(EIMSK, 6);

    return (char *)vs_writeptr;
}

/*!
 * \brief Returns total free buffer space.
 *
 * \return Total number of free bytes in the buffer.
 */
u_short VsBufferTotalFree(void)
{
    u_char *loc_vs_readptr=(char *)vs_readptr; //local copy of volatile variable
    if (vs_writeptr>=loc_vs_readptr)
      return (vs_dataend-vs_databuff)-(vs_writeptr-loc_vs_readptr)-1;
    else
      return (loc_vs_readptr-vs_writeptr)-1;
}


/*!
 * \brief Set volume.
 *
 * \param left  Left channel volume.
 * \param right Right channel volume.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsSetVolume(u_char left, u_char right)
{
    VsRegWrite(VS_VOL_REG, (((u_short) left) << 8) | (u_short) right);
    return 0;
}

/*!
 * \brief Sine wave beep.
 *
 * \param fsin Frequency.
 * \param ms   Duration.
 *
 * \return 0 on success, -1 otherwise.
 */
int VsBeep(u_char fsin, u_char ms)
{
    u_char i;
    u_char on[] = { 0x53, 0xEF, 0x6E, 56 };
    u_char off[] = { 0x45, 0x78, 0x69, 0x74 };

    on[3] += (fsin & 7) * 9;
    for (i = 0; i < sizeof(on); i++)
        VsSdiPutByte(on[i]);
    for (i = 0; i < 4; i++)
        VsSdiPutByte(0);
    NutDelay(ms);
    for (i = 0; i < sizeof(off); i++)
        VsSdiPutByte(off[i]);
    for (i = 0; i < 4; i++)
        VsSdiPutByte(0);

    return 0;
}


/*@}*/
